#!/bin/bash
configDir=/etc/azuresecurity
# Audit 11
chmod 400 /etc/*shadow
# (Audit 105) Remove unnecessary packages
apt-get remove -y at landscape-common
rm -fr /etc/landscape /var/log/landscape
# (Audit 106, 109, 113) Update ssh policy
sed -i 's/^[# ]*\(Protocol\s\+\)[1-2\,]\+/\12/g' /etc/ssh/ssh_config
sed -i 's/^[# ]*\(RhostsRSAAuthentication\s\+\)[a-z]\+/\1no/g' /etc/ssh/sshd_config
sed -i 's/^[# ]*\(PermitRootLogin\s\+\)[a-z]\+/\1no/g' /etc/ssh/sshd_config
echo "Ciphers aes128-ctr,aes192-ctr,aes256-ctr" >> /etc/ssh/sshd_config
/etc/init.d/ssh restart
# (Audit 18.a) Fix umask for daemons
sed -i 's/^[#\s]*\(umask\s\+\)[0-9]\{3\}/\1027/g' /etc/init/*.conf
egrep -L 'task|umask' /etc/init/*.conf* | grep -v resolv | xargs sed -i 's/^start/umask 027\nstart/g'
sed -i 's/^start/umask 022\nstart/g' /etc/init/resolvconf.conf
sed -i 's/^start/umask 027\nstart/g' /etc/init/rcS.conf
# (Audit 20) Restrict root login
echo -e "console\ntty1" > /etc/securetty
# (Audit 21) Fix su permissions
sed -i 's/^#*\s*\(auth\s\+required\s\+pam_wheel.so$\)/\1 use_uid/g' /etc/pam.d/su
# (Audit 28) Update 'home' folder permissions
chmod 750 /home/*
chmod 750 /var/lib/libuuid
chmod 750 /var/run/dbus
chmod 750 /var/lib/clamav
# /var/run/sshd created by service at bootup
sed -i 's/\(chmod\s\+\)[0-7]\{4\}/\10750/g' /etc/init.d/ssh
sed -i 's/\(mkdir\s\+-p\s\+-m\)[0-9]\{4\}/\10750/g' /etc/init/ssh.conf
# (Audit 29) Set default umask to 077
sed -i 's/^\(UMASK\s\+\)[0-9]\{3\}/\1077/g' /etc/login.defs
# (Audit 30/32) Set grub modules to readable only by root
find /boot /boot/grub -maxdepth 1 -type f | xargs chmod 600
chmod 600 /etc/default/grub*
chmod go-rwx /etc/grub.d/*
# (Audit 31,34) Enable grub password for edit, console, and recovery mode
pass="$RANDOM$RANDOM$RANDOM"
encPass=`printf '%s\n%s\n' $pass $pass | grub-mkpasswd-pbkdf2 | grep grub | awk '{printf $4}'`
superUser=root
printf 'set superusers="%s"\npassword_pbkdf2 %s %s\n' $superUser $superUser $encPass >> /etc/grub.d/40_custom
sed -i 's/\(CLASS\}\)\s*\(.*GRUB_RECOVERY_TITLE\)/\1 --users \\\"\\\" \2/g' /etc/grub.d/10_linux
echo 'chmod 400 ${grub_cfg}' >> /usr/sbin/grub-mkconfig
update-grub
# (Audit 111) Set SSH login banner
sed -i 's/[#\s]*\(Banner\).*$/\1 \/etc\/issue.net/g' /etc/ssh/sshd_config
# (Audit 38/39/41/51.1-8, 42.1, 45, 60) Disable IP Forwarding, enable reverse path and tcp SYN cookies
sed -i 's/^#*\s*\(net.ipv[46].ip_forward\s*=\s*\)[0-9]/\10/g' /etc/sysctl.conf
sed -i 's/^#*\s*\(net.ipv[46].tcp_syncookies\s*=\s*\)[0-9]/\11/g' /etc/sysctl.conf
sed -i 's/^#*\s*\(net.ipv[46].conf.[default]*.rp_filter\s*=\s*\)[0-9]/\11/g' /etc/sysctl.conf
# (Audit 38/39/41/51.1-8, 42.1, 45, 60) Handle Redirects and Martians
sed -i 's/^#*\s*\(net.ipv[46].conf.all.accept_redirects\s*=\s*\)[0-9]/\10/g' /etc/sysctl.conf
sed -i 's/^#*\s*\(net.ipv[46].conf.all.secure_redirects\s*=\s*\)[0-9]/\10/g' /etc/sysctl.conf
sed -i 's/^#*\s*\(net.ipv[46].conf.all.send_redirects\s*=\s*\)[0-9]/\10/g' /etc/sysctl.conf
sed -i 's/^#*\s*\(net.ipv[46].conf.all.accept_source_route\s*=\s*\)[0-9]/\10/g' /etc/sysctl.conf
sed -i 's/^#*\s*\(net.ipv[46].conf.all.log_martians\s*=\s*\)[0-9]/\11/g' /etc/sysctl.conf
# (Audit 38/39/41/51.1-8, 42.1, 45, 60) Insert default entries where existing entries are found
sed -i 's/\(net.ipv[46].conf.\)all\(.accept_redirects\s*=\s*\)\([0-9]\)/\1all\2\3\n\1default\20/g' /etc/sysctl.conf
sed -i 's/\(net.ipv[46].conf.\)all\(.secure_redirects\s*=\s*\)\([0-9]\)/\1all\2\3\n\1default\20/g' /etc/sysctl.conf
sed -i 's/\(net.ipv[46].conf.\)all\(.send_redirects\s*=\s*\)\([0-9]\)/\1all\2\3\n\1default\20/g' /etc/sysctl.conf
sed -i 's/\(net.ipv[46].conf.\)all\(.accept_source_route\s*=\s*\)\([0-9]\)/\1all\2\3\n\1default\20/g' /etc/sysctl.conf
sed -i 's/\(net.ipv[46].conf.\)all\(.log_martians\s*=\s*\)\([0-9]\)/\1all\2\3\n\1default\21/g' /etc/sysctl.conf
echo "net.ipv4.icmp_ignore_bogus_error_responses=1" >> /etc/sysctl.conf
echo "net.ipv4.icmp_echo_ignore_broadcasts=1" >> /etc/sysctl.conf
sysctl -p /etc/sysctl.conf
# Require client SMB signing
sed -i "s/\[global\]/\[global\]\nclient signing = mandatory/" /etc/samba/smb.conf
# (Audit 92) Fix Permissions on cron files/folders
chmod 600 /etc/crontab
# (Audit 93-97, 100-103) Fix Permissions on cron files/folders
chmod 700 /etc/cron.*
# (Audit 119) Configure ntpd
sed -i 's/^\([\s]*server\s\+\)0.*$/\1time.windows.com/g' /etc/ntp.conf
sed -i '/^[\s]*server\s\+[1-3].*$/d' /etc/ntp.conf
/etc/init.d/ntp reload
# Clean up non-root system accounts
userdel games
userdel landscape
# (Audit 23.1) Disable login for system accounts
for i in `egrep -v '^([^:]*:){5}.*(root|home)' /etc/passwd | cut -d ":" -f 1; echo syslog`
do 
    chsh -s /bin/false $i
done
# Set firewall rules. Ping, loopback, and established connections allowed by default. Default rule is DROP
ufw allow ssh
ufw logging low
echo y | ufw enable
# (Audit 15/16)Update SUID and SGID permissions
chmod ug-s `cat $configDir/removeSpecialFilePermissions.txt`
# (Audit6.1 - 6.6, 54, 55, 57) Disable unnecessary Kernel Modules
cat $configDir/blacklist-azurebaseline.conf > /etc/modprobe.d/blacklist-azurebaseline.conf
chmod 644 /etc/modprobe.d/blacklist-azurebaseline.conf
# (Audit 157/1-8) Update Password Policy
sed -i 's/^#*\s*\(password\s\+requisite\s\+pam_cracklib.so\)/\1 ucredit=-1 lcredit=-2 dcredit=-1 ocredit=-1/g' /etc/pam.d/common-password
sed -i 's/\(minlen=\)[0-9]\+/\110/g' /etc/pam.d/common-password
sed -i 's/^#*\s*\(password\s\+.\+pam_unix.so\)/\1 remember=7/g' /etc/pam.d/common-password
sed -i 's/^#*\s*\(PASS_MAX_DAYS\s\+\)[0-9]\+/\170/g' /etc/login.defs
sed -i 's/^#*\s*\(PASS_WARN_AGE\s\+\)[0-9]\+/\115/g' /etc/login.defs
