#!/usr/bin/python
#
# Copyright 2014 Microsoft Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import imp
import os
import os.path
import platform
import re
import syslog

defaultFacility = syslog.LOG_AUTH
configDir="/etc/azuresecurity"
scriptDir=os.path.dirname(os.path.realpath(__file__))
ubuntu = "Ubuntu"
runtimeVerbosity = 0
enabled = "Enabled"
disabled = "Disabled"

# Return true if the string 'constState' starts with 'proposedState' in a case insensitive match
def IsConfigMatch(proposedState, constState):
    if not (proposedState or constState):
        return false
    return constState.lower().startswith(proposedState.lower())

# Return true if the string 'enabled' starts with the specified string in a case insensitive match
def IsEnabled(proposedState):
    return IsConfigMatch(proposedState, enabled)
    
# Return true if the string 'disabled' starts with the specified string in a case insensitive match
def IsDisabled(proposedState):
    return IsConfigMatch(proposedState, disabled)

# The following code will search and load waagent code and expose
# it as a submodule of current module
def searchWAAgent():
    agentPath = '/usr/sbin/waagent'
    if(os.path.isfile(agentPath)):
        return agentPath
    user_paths = os.environ['PYTHONPATH'].split(os.pathsep)
    for user_path in user_paths:
        agentPath = os.path.join(user_path, 'waagent')
        if(os.path.isfile(agentPath)):
            return agentPath
    return None

# Define the function in case waagent(<2.0.4) doesn't have DistInfo()
def DistInfo(fullname=0):
    if 'FreeBSD' in platform.system():
        release = re.sub('\-.*\Z', '', str(platform.release()))
        distinfo = ['FreeBSD', release]
        return distinfo
    if os.path.isfile('/etc/oracle-release'):
        release = re.sub('\-.*\Z', '', str(platform.release()))
        distinfo = ['Oracle', release]
        return distinfo
    if 'linux_distribution' in dir(platform):
        distinfo = list(platform.linux_distribution(\
                        full_distribution_name=fullname))
        # remove trailing whitespace in distro name
        distinfo[0] = distinfo[0].strip()
        return distinfo
    else:
        return platform.dist()
       
# 0 = Default message verbosity (-v), write to syslog
# 1 = Verbose output (-vv), never write to syslog
# 2 = Debug output (-vvv), never write to syslog
def WriteLog(level, message, messageVerbosity=0):
    if (len(message) < 0):
        return
    if (messageVerbosity < 1):
        syslog.syslog(level, message)
    if (messageVerbosity < runtimeVerbosity):
        print message

# Writes the specified message with a verbosity that is guaranteed to write to console
def WriteConsole(level, message):
    WriteLog(level, message, -1)

# Writes the specified message with verbosity level 1 (-vv)
def WriteVerbose(message):
    WriteLog(syslog.LOG_INFO, message, 1)

# Writes the specified message with verbosity level 2 (-vvv)
def WriteDebug(message):
    WriteLog(syslog.LOG_INFO, message, 2)
    
distro = DistInfo()[0]
agentPath = searchWAAgent()
if(agentPath):
    waagent = imp.load_source('waagent', agentPath)
else:
    raise Exception("Can't load waagent.")
